@@ -1,7 +1,7 @@
 /*
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2017
+    written by Marc Ihm 1995-2018
     more info at www.yabasic.de
 
     flow.c --- code for subroutines and flow-control
@@ -205,6 +205,7 @@ create_subr_link (char *label)	/* create link to subroutine */
     struct command *cmd;
 
     if (!inlib) {
+	error(DEBUG, "not in library, will not create link to subroutine");
         return;
     }
     dot = strchr (label, '.');
@@ -386,7 +387,7 @@ add_switch_state(struct command *cmd) /* add switch state to a newly created com
     cmd->switch_state = my_malloc (sizeof (struct switch_state));
     cmd->switch_state->id = switch_id_stack[switch_nesting];
     cmd->switch_state->nesting = switch_nesting;
-    cmd;
+    return cmd;
 }
 
 
@@ -617,14 +618,15 @@ create_label (char *label, int type)	/* creates command label */
 
 
 void
-decide ()			/*  skips next command, if not 0 on stack */
+decide()			/*  skips next command, if not 0 on stack */
 {
-    if (pop (stNUMBER)->value != 0) {
-        current = current->next;    /* skip one command */
-	std_diag ("skipping", current->type, current->symname, current->diag);
-    } else {
-	if (infolevel >= DEBUG)	error (DEBUG, "(no command skipped)");
-    }
+	if (pop(stNUMBER)->value != 0) {
+		current = current->next;    /* skip one command */
+		if (infolevel >= DEBUG) std_diag("skipping", current->type, current->symname, current->diag);
+	}
+	else {
+		if (infolevel >= DEBUG)	error(DEBUG, "(no command skipped)");
+	}
 }
 
 
