@@ -1,7 +1,7 @@
 /*
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2017
+    written by Marc Ihm 1995-2018
     more info at www.yabasic.de
 
     function.c --- code for functions
@@ -1500,6 +1500,7 @@ static double
 peek (char *dest)		/* peek into internals */
 {
     char *s;
+    time_t now;
 
     for (s = dest; *s; s++) {
         *s = tolower ((int) *s);
@@ -1522,6 +1523,9 @@ peek (char *dest)		/* peek into internals */
         return errorcode;
     } else if (!strcmp (dest, "isbound")) {
         return is_bound;
+    } else if (!strcmp (dest, "secondsrunning")) {
+	time(&now);
+	return now-compilation_start;
     } else if (dest[0] == '#') {
         error (ERROR, "don't use quotes when peeking into a file");
         return 0;
@@ -1575,6 +1579,10 @@ peek2 (char *dest, struct command *curr)	/* peek into internals */
         return my_strdup (winorigin);
     } else if (!strcmp (dest, "error")) {
         return my_strdup (errorstring);
+    } else if (!strcmp (dest, "program_file_name")) {
+        return my_strdup(main_file_name);
+    } else if (!strcmp (dest, "program_name")) {
+        return my_strdup(progname);
     } else if (!strcmp (dest, "library")) {
         return my_strdup (curr->lib->s);
     } else if (!strcmp (dest, "version")) {
