@@ -1,7 +1,7 @@
 /*
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2017
+    written by Marc Ihm 1995-2018
     more info at www.yabasic.de
 
     io.c --- code for screen and file i/o
@@ -135,7 +135,7 @@ print(struct command *cmd)	/* print on screen */
     case 'n':			/* print newline */
         if (curinized && coutstr == stdout) {
 #ifdef WINDOWS
-            onestring("\r\n");
+            onestring ("\r\n");
             break;
 #else
             getyx (stdscr, y, x);
@@ -1287,7 +1287,7 @@ create_onestring(char *str)	/* create command 'onestring' */
 
 
 void
-onestring(char *s)		/* write string to file */
+onestring (char *s)		/* write string to current file */
 {
 #ifdef WINDOWS
     DWORD len, written;
