@@ -1,7 +1,7 @@
 /*
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2017
+    written by Marc Ihm 1995-2018
     more info at www.yabasic.de
 
     main.c --- main() and auxilliary functions
@@ -21,10 +21,10 @@
 /* ------------- defines ---------------- */
 
 #define DONE {current=current->next;break;}	/* reduces type-work */
-#define COPYRIGHT "Copyright 1995-2017 by Marc Ihm, according to the MIT License"
+#define COPYRIGHT "Copyright 1995-2018 by Marc Ihm, according to the MIT License"
 #define BANNER \
-"\nThis is yabasic version " VERSION ",\nbuilt on "\
-ARCHITECTURE " at " BUILD_TIME "\n\n   " COPYRIGHT "\n\n"
+"\nThis is yabasic version " VERSION ",\ncompiled on "\
+ARCHITECTURE " at " __TIMESTAMP__ "\n\n   " COPYRIGHT "\n\n"
 #define BANNER_VERSION \
 "yabasic " VERSION ", built on " ARCHITECTURE
 #define YABFORHELP "(type 'yabasic --help' for help)"
@@ -48,7 +48,10 @@ static void chop_command (char *, int *, char ***);	/* chops WIN95-commandline *
 void create_docu_array (void);	/* create array with documentation */
 int equal (char *, char *, int);	/* helper for processing options */
 static int mybind (char *);	/* bind a program to the interpreter and save it */
+void put_and_count (char *, FILE *, int *);  /* write text to file and increment len */
+
 char *find_interpreter (char *);	/* find interpreter with full path */
+static int seekback (FILE *, int);       /* seek back bytes */
 
 /* ------------- global variables ---------------- */
 
@@ -91,8 +94,8 @@ char library_default[200];	/* default full path to search libraries */
 static struct command *docuhead = NULL;	/* first docu in main */
 static int docucount = 0;	/* number of docu-lines in array */
 int check_compat = 0;		/* true, if compatibility should be checked */
-static char *interpreter_path = NULL;	/* name of interpreter executing; i.e. ARGV[0] */
-static char *main_file_name = NULL;	/* name of program to be executed */
+char *inter_path = NULL;	/* name of interpreter executing; i.e. ARGV[0] */
+char *main_file_name = NULL;	/* name of program to be executed */
 
 /* ------------- main program ---------------- */
 
@@ -180,7 +183,7 @@ main (int argc, char **argv)
 #endif
 
     /* find out, if this executable is bound to a yabasic program */
-    interpreter_path = find_interpreter (argv[0]);
+    inter_path = find_interpreter (argv[0]);
     is_bound = isbound ();
 
     /* parse arguments */
@@ -209,7 +212,7 @@ main (int argc, char **argv)
     time (&compilation_start);
     last_inkey=my_malloc(INBUFFLEN);
     last_inkey[0]='\0';
-    error (DEBUG, "this is yabasic " VERSION);
+    error (DEBUG, "This is yabasic " VERSION ", compiled on " ARCHITECTURE " at " __TIMESTAMP__ ", configured at " BUILD_TIME);
     initialize ();
     program_state = INITIALIZED;
 
@@ -250,12 +253,12 @@ main (int argc, char **argv)
     if (to_bind) {
         if (mybind (to_bind)) {
             sprintf (string, "Successfully bound '%s' and '%s' into '%s'",
-                     interpreter_path, main_file_name, to_bind);
+                     inter_path, main_file_name, to_bind);
             error (INFO, string);
             end_it ();
         } else {
             sprintf (string, "Could not bind '%s' and '%s' into '%s'",
-                     interpreter_path, main_file_name, to_bind);
+                     inter_path, main_file_name, to_bind);
             error (ERROR, string);
             end_it ();
         }
@@ -263,6 +266,10 @@ main (int argc, char **argv)
 
     if (errorlevel > ERROR && !check_compat) {
         program_state = RUNNING;
+	if (infolevel >= DEBUG) {
+	    printf ("---Program parsed, press RETURN to continue with its execution: ");
+	    fgets (string, 2, stdin);
+        }
         run_it ();
     } else {
         program_state = FINISHED;
@@ -446,7 +453,7 @@ add_command_with_switch_state (int type)	/* same as add_command, but add switch_
     struct command *cmd;
 
     cmd = add_command (type, NULL, NULL);
-    add_switch_state(cmd);
+    return add_switch_state(cmd);
 }
 
 
@@ -777,7 +784,7 @@ parse_arguments (int cargc, char *cargv[])
 
     if (is_bound) {
         inputfile = bound_program;
-        main_file_name = my_strdup (interpreter_path);
+        main_file_name = my_strdup (inter_path);
     } else if (!inputfile && !explicit) {
         interactive = TRUE;
         inputfile = stdin;
@@ -1316,7 +1323,7 @@ run_it ()
                 l++;
                 if (hold_docu && !(l % 24)) {
                     printf ("---Press RETURN to continue ");
-                    fgets (string, 20, stdin);
+                    fgets (string, 2, stdin);
                 }
             } else {
                 if (infolevel >= DEBUG) {
@@ -1330,7 +1337,7 @@ run_it ()
         }
         if (hold_docu) {
             printf ("---End of embbedded documentation, press RETURN ");
-            fgets (string, 20, stdin);
+            fgets (string, 2, stdin);
         }
     } else {
         while (current != cmdhead && endreason == erNONE) {
@@ -1888,7 +1895,7 @@ new_file (char *l, char *s)  	/* create a new structure for library names */
     if (s) {
         new->s = my_strdup (s);
     } else {
-        /* no short name supplied get piece from l */
+        /* no short name supplied; get piece from l */
         end = strlen (l);
         for (start = end; start > 0; start--) {
             if (l[start - 1] == '\\' || l[start - 1] == '/') {
@@ -2089,85 +2096,147 @@ create_docu_array (void)	/* create array with documentation */
 int
 isbound (void)			/* check if this interpreter is bound to a program */
 {
-    FILE *interpreter;
+    FILE *inter;
     int i;
     int c;
     int proglen = 0;
+    int namelen = 0;
     int bound = 1;
+    char *infolevel_text;
+    int remlen = strlen("\nrem "); /* different under windows and unix */
+    int offset = 0;
 
-    if (!interpreter_path || !interpreter_path[0]) {
-        error (FATAL, "interpreter_path is not set !");
+    if (!inter_path || !inter_path[0]) {
+        error (FATAL, "inter_path is not set !");
         return 0;
     }
-    if (!(interpreter = fopen (interpreter_path, "r"))) {
+    if (!(inter = fopen (inter_path, "r"))) {
         sprintf (string, "Couldn't open '%s' to check, if it is bound: %s",
-                 interpreter_path, my_strerror (errno));
+                 inter_path, my_strerror (errno));
         error (WARNING, string);
         return 0;
     }
 
-    if (fseek (interpreter, 0 - strlen (YABMAGIC) - 1, SEEK_END)) {
-        sprintf (string, "Couldn't seek within '%s': %s", interpreter_path,
-                 my_strerror (errno));
-        error (WARNING, string);
-        return 0;
-    }
+    /* Read fields from end of program as written in mybind() */
+
+    /* check magic cookie */
+    offset -= 1 + strlen (YABMAGIC);
+    if (!seekback (inter, offset)) return 0;
     for (i = 0; i < (int) strlen (YABMAGIC); i++) {
-        c = fgetc (interpreter);
+        c = fgetc (inter);
         if (c == EOF || c != (YABMAGIC)[i]) {
             bound = 0;
         }
     }
     if (!bound) {
-        fclose (interpreter);
+        fclose (inter);
         return bound;
     }
 
-    if (fseek (interpreter, 0 - strlen (YABMAGIC) - 5 - 8 - 1, SEEK_END)) {
-        sprintf (string, "Couldn't seek within '%s': %s", interpreter_path,
-                 my_strerror (errno));
-        error (WARNING, string);
+    /* infolevel */
+    offset -= remlen + 2;
+    if (!seekback (inter, offset)) return 0;
+    if (!fscanf (inter, "%d", &infolevel)) {
+        error (WARNING, "Could not read infolevel");
         return 0;
     }
-    if (!fscanf (interpreter, "%d", &proglen)) {
+    /* repeat just for its output side-effect */
+    if (!seekback (inter, offset)) return 0; 
+    switch(infolevel) {
+    case FATAL: infolevel_text="FATAL";break;
+    case ERROR: infolevel_text="ERROR";break;
+    case INFO: infolevel_text="INFO";break;
+    case DUMP: infolevel_text="DUMP";break;
+    case WARNING: infolevel_text="WARNING";break;
+    case NOTE: infolevel_text="NOTE";break;
+    case DEBUG: infolevel_text="DEBUG";break;
+    case DEBUG+1: infolevel_text="DEBUG+BISON";yydebug=1;infolevel=DEBUG;break;};
+    sprintf (errorstring, "Set infolevel to %s", infolevel_text);
+    error (DEBUG, errorstring);
+
+    /* length of name of embedded program */
+    offset -= remlen + 8;
+    if (!seekback (inter, offset)) return 0;
+    if (!fscanf (inter, "%d", &namelen)) {
+        error (WARNING, "Could not read length of name of embedded program");
+        return 0;
+    }
+    sprintf (errorstring, "Length of name of embedded program is %d", namelen);
+    error (DEBUG, errorstring);
+    
+    /* name of embedded program */
+    offset -= remlen + namelen;
+    if (!seekback (inter, offset)) return 0;
+    progname = (char *) my_malloc (sizeof (char) * (namelen + 1));
+    if (!fgets (progname, namelen + 1, inter)) {
+        error (WARNING, "Could not read name of embedded program");
+        return 0;
+    }
+    sprintf (errorstring, "Name of embedded program is '%s'", progname);
+    error (DEBUG, errorstring);
+
+    /* length of program */
+    offset -= remlen + 8;
+    if (!seekback (inter, offset)) return 0;
+    if (!fscanf (inter, "%d", &proglen)) {
         error (WARNING, "Could not read length of embedded program");
         return 0;
     }
+    sprintf (errorstring, "Length of embedded program is %d", proglen);
+    error (DEBUG, errorstring);
 
-    if (fseek
-            (interpreter, 0 - strlen (YABMAGIC) - 5 - 8 - 5 - 5 - proglen,
-             SEEK_END)) {
-        sprintf (string, "Couldn't seek within '%s': %s", interpreter_path,
-                 my_strerror (errno));
-        error (WARNING, string);
-        return 0;
-    }
+    /* seek back to start of embedded program */
+    offset -= 4 + proglen; /* only the text 'rem ' without preceding linefeed */
+    if (!seekback (inter, offset)) return 0;
 
     if (infolevel >= NOTE) {
         error (NOTE, "Dumping the embedded program, that will be executed:");
         fprintf (stderr, "     ");
         for (i = 0; i < proglen; i++) {
-            c = fgetc (interpreter);
-            fprintf (stderr, "%c", c);
+            c = fgetc (inter);
+            if (isprint(c) || c == '\r' || c== '\n') fprintf (stderr, "%c", c);
             if (c == '\n' && i < proglen - 1) {
                 fprintf (stderr, "     ");
             }
         }
+	fprintf (stderr, "\n");
         error (NOTE, "End of program, that will be executed");
-        if (fseek
-                (interpreter, 0 - strlen (YABMAGIC) - 5 - 8 - 5 - 5 - proglen,
-                 SEEK_END)) {
-            sprintf (string, "Couldn't seek within '%s': %s",
-                     interpreter_path, my_strerror (errno));
-            error (WARNING, string);
-            return 0;
-        }
+	printf ("---Press RETURN to continue with its parsing: ");
+	fgets (string, 2, stdin);
+	if (!seekback (inter, offset)) return 0;
     }
-    bound_program = interpreter;
+    bound_program = inter;
     return 1;
 }
 
 
+static int
+seekback (FILE *file, int offset)           /* seek back bytes */
+{
+  if (fseek (file, offset, SEEK_END)) {
+    sprintf (errorstring, "Couldn't seek within '%s': %s", inter_path,
+	     my_strerror (errno));
+    error (WARNING, errorstring);
+    return FALSE;
+  }
+  if (!fgets (string, INBUFFLEN, file)) {
+    error (WARNING, "Could not read from end of embedded program");
+    return FALSE;
+  }
+  string[strlen(string) - strlen("\n")] = '\0';
+  sprintf(errorstring, "Next line from end of embbeded program to be processed is: '%s'", string);
+  error (DEBUG, errorstring);
+  if (fseek (file, offset, SEEK_END)) {
+    sprintf (errorstring, "Couldn't seek within '%s': %s", inter_path,
+	     my_strerror (errno));
+    error (WARNING, errorstring);
+    return FALSE;
+  }
+    
+  return TRUE;
+}
+
+
 static int
 mybind (char *bound)		/* bind a program to the interpreter and save it */
 {
@@ -2176,18 +2245,17 @@ mybind (char *bound)		/* bind a program to the interpreter and save it */
     FILE *fbound;
     FILE *flib;
     int c;
-    char *pc;
     int i;
     int proglen = 0;
 
     if (interactive) {
-        error (ERROR, "cannot bind a program when called interactive");
+        error (ERROR, "cannot bind a program when called interactively");
         return 0;
     }
 
-    if (!strcmp (interpreter_path, bound)) {
+    if (!strcmp (inter_path, bound)) {
         sprintf (string, "will not overwrite '%s' with '%s'", bound,
-                 interpreter_path);
+                 inter_path);
         error (ERROR, string);
         return 0;
     }
@@ -2198,9 +2266,9 @@ mybind (char *bound)		/* bind a program to the interpreter and save it */
         return 0;
     }
 
-    if (!(fyab = fopen (interpreter_path, "rb"))) {
+    if (!(fyab = fopen (inter_path, "rb"))) {
         sprintf (string, "could not open '%s' for reading: %s",
-                 interpreter_path, my_strerror (errno));
+                 inter_path, my_strerror (errno));
         error (ERROR, string);
         return 0;
     }
@@ -2221,7 +2289,7 @@ mybind (char *bound)		/* bind a program to the interpreter and save it */
     }
 
     if (infolevel >= DEBUG) {
-        sprintf (string, "binding %s and %s into %s", interpreter_path,
+        sprintf (string, "binding %s and %s into %s", inter_path,
                  main_file_name, bound);
         error (NOTE, string);
     }
@@ -2238,30 +2306,29 @@ mybind (char *bound)		/* bind a program to the interpreter and save it */
             return 0;
         }
         sprintf (string, "\nimport %s\n", libfile_chain[i]->s);
-        for (pc = string; *pc; pc++) {
-            fputc (*pc, fbound);
-            proglen++;
-        }
+	put_and_count(string, fbound, &proglen);
+	
+	put_and_count("\nimport __IGNORE_NESTED_IMPORTS\n", fbound, &proglen);
         while ((c = fgetc (flib)) != EOF) {
             fputc (c, fbound);
             proglen++;
         }
+	
+	put_and_count("\nimport __END_OF_CURRENT_IMPORT\n", fbound, &proglen);
     }
 
-    for (pc = "\nimport main\n"; *pc; pc++) {
-        fputc (*pc, fbound);
-        proglen++;
-    }
-    for (pc = "\nimport __END_OF_IMPORT\n"; *pc; pc++) {
-        fputc (*pc, fbound);
-        proglen++;
-    }
+    put_and_count("\nimport __END_OF_ALL_IMPORTS\n", fbound, &proglen);
+    put_and_count("\nimport main\n", fbound, &proglen);
     while ((c = fgetc (fprog)) != EOF) {
         fputc (c, fbound);
         proglen++;
     }
     fprintf (fbound, "\nend\n");
+    proglen += 5;
     fprintf (fbound, "rem %08d\n", proglen);
+    fprintf (fbound, "rem %s\n", progname);
+    fprintf (fbound, "rem %08d\n", strlen(progname));
+    fprintf (fbound, "rem %02d\n", infolevel + yydebug);
     fprintf (fbound, "rem %s\n", YABMAGIC);
     fclose (fyab);
     fclose (fprog);
@@ -2270,6 +2337,17 @@ mybind (char *bound)		/* bind a program to the interpreter and save it */
     return 1;
 }
 
+
+void
+put_and_count (char *text, FILE *file, int *len)  /* write text to file and increment len */
+{
+    for (; *text; text++) {
+        fputc (*text, file);
+        (*len)++;
+    }
+}
+
+
 char *
 find_interpreter (char *name)	/* find interpreter with full path */
 {
